@@ -1,9 +1,6 @@
 '''datapkg command line interface.
 
-datapkg ondiskorpath create | dump | info | register | install
-datapkg repo init | list | sync | search
-          index list | sync | search
-datapkg doc about | license | man
+See datapkg help for details.
 '''
 import sys
 import os
@@ -18,6 +15,7 @@ logger = logging.getLogger('datapkg.cli')
 logging.basicConfig()
 
 import datapkg
+import datapkg.index
 
 parser = optparse.OptionParser(
     usage='''%prog COMMAND [OPTIONS]
@@ -42,16 +40,6 @@ parser.add_option(
     dest='log',
     metavar='FILENAME',
     help='Log file where a complete (maximum verbosity) record will be kept')
-parser.add_option(
-    '-r', '--repository',
-    dest='repository',
-    help='Path to repository (if non-default)',
-    default=None)
-parser.add_option(
-    '-k', '--api-key',
-    dest='api_key',
-    default=None,
-    help='API Key for use with repository (where necessary)')
 # TODO: this should be made specific to CreateCommand
 # problem is how to have general parser and specific parser ...
 # http://osdir.com/ml/python.optik.user/2008-02/msg00001.html
@@ -60,6 +48,23 @@ parser.add_option(
     dest='template',
     default='default',
     help='Specify a template to use when creating on disk (default or flat)')
+# TODO: put in defaults for repo and config
+import datapkg.config
+parser.add_option(
+    '-c', '--config',
+    dest='config',
+    help='Path to config file (if any) - defaults to %default',
+    default=datapkg.config.default_config_path)
+parser.add_option(
+    '-r', '--repository',
+    dest='repository',
+    help='Path to repository - overrides value in config'
+    )
+parser.add_option(
+    '-k', '--api-key',
+    dest='api_key',
+    default=None,
+    help='CKAN API Key (overrides value in config)')
 
 _commands = {}
 
@@ -87,46 +92,72 @@ class Command(object):
         options.verbose += initial_options.verbose
 
     def _print(self, msg, force=False):
-        if self.verbose or force:
+        if self.level >= 1 or force:
             print(msg)
 
-    def _get_repo(self):
-        import datapkg.repository
-        # HACK: fix this up
-        if self.repository_path and 'http://' in self.repository_path:
-            class StubbedRepo(object):
-                pass
-            repo = StubbedRepo()
-            repo.index = datapkg.index.CkanIndex(
-                    rest_api_url=self.repository_path,
-                    api_key=self.options.api_key)
-        else:
-            repo = datapkg.repository.Repository(self.repository_path)
-        return repo
+    @classmethod
+    def parse_spec(self, spec=None):
+        '''
+        @params spec: if None default to file://
+        '''
+        import urlparse
+        if spec is None:
+            spec = 'file://'
+        scheme, netloc, path, query, fragment = urlparse.urlsplit(spec)
+        # case where we just provide a path ...
+        if scheme == '':
+            scheme = 'file'
+        if scheme == 'file':
+            # for file netloc is everything up to last name
+            netloc = os.path.join(netloc, os.path.dirname(path))
+        if scheme == 'ckan':
+            # deal with preceding slashes in ckan://...
+            while path.startswith('/'):
+                path = path[1:]
+            netloc = '/'.join(path.split('/')[:-1])
+            # we have a path but did not put http:// ...
+            if netloc and not netloc.startswith('http'):
+                netloc = 'http://' + netloc
+            path = path.split('/')[-1]
+        return scheme, netloc, path
 
-    def _register(self, path):
-        import datapkg.package
-        pkg = datapkg.package.Package.from_path(path)
-        repo = self._get_repo()
-        repo.index.register(pkg)
-        return (repo, pkg)
+    def index_from_spec(self, spec=None):
+        '''Load an `Index` from a spec.
 
-    def _get_package(self, path_or_name):
-        path_or_name = unicode(path_or_name)
-        import datapkg.package
-        # this is crude
-        is_path = os.path.exists(path_or_name)
-        if is_path:
-            return datapkg.package.Package.from_path(path_or_name)
+        @return: `Index` and path
+
+        schemes = [
+            'file',
+            'ckan',
+            'db',
+            ]
+        '''
+        scheme, netloc, path = self.parse_spec(spec)
+        if scheme == 'file':
+            # for file netloc is everything up to last name
+            netloc = os.path.join(netloc, os.path.dirname(path))
+            index = datapkg.index.FileIndex(netloc)
+        elif scheme == 'ckan':
+            api_key = self.options.api_key
+            if not api_key:
+                api_key = self._config.get('DEFAULT', 'ckan.api_key')
+            if netloc:
+                ckan_url = netloc
+            else:
+                ckan_url = self._config.get('DEFAULT', 'ckan.url')
+            index = datapkg.index.CkanIndex(
+                    rest_api_url=ckan_url,
+                    api_key=api_key)
         else:
-            repo = self._get_repo()
-            return repo.index.get(path_or_name)
+            msg = 'Scheme "%s" not recognized' % scheme
+            raise Exception(msg)
+        return index, path
 
     def _print_pkg(self, pkg):
-        out = StringIO()
         print u'## Package: %s' % pkg.name
         print
-        print pkg.metadata.pprint()
+        out = pkg.pretty_print()
+        print out
 
     def main(self, complete_args, args, initial_options):
         options = initial_options
@@ -136,11 +167,17 @@ class Command(object):
         self.options = options
         self.repository_path = options.repository
         self.verbose = options.verbose
+        import datapkg.config
+        self._config = datapkg.config.get_config(options.config)
+        if not self.repository_path:
+            self.repository_path = self._config.get('DEFAULT', 'repo.default_path')
+        
 
         # TODO: fix up logger
         level = 1 # Notify
         level += options.verbose
         level -= options.quiet
+        self.level = level
         complete_log = []
         if options.log:
             log_fp = open_logfile_append(options.log)
@@ -158,13 +195,16 @@ class Command(object):
         if log_fp is not None:
             log_fp.close()
         if exit:
-            log_fn = 'datapkg-log.txt'
-            text = '\n'.join(complete_log)
-            # Not sure we need to tell people ...
-            # logger.fatal('Storing complete log in %s' % log_fn)
-            log_fp = open_logfile_append(log_fn)
-            log_fp.write(text)
-            log_fp.close()
+            try:
+                log_fn = 'datapkg-log.txt'
+                text = '\n'.join(complete_log)
+                # Not sure we need to tell people ...
+                # logger.fatal('Storing complete log in %s' % log_fn)
+                log_fp = open_logfile_append(log_fn)
+                log_fp.write(text)
+                log_fp.close()
+            except IOError:
+                pass
         sys.exit(exit)
 
 
@@ -250,34 +290,66 @@ class ListCommand(Command):
     name = 'list'
     summary = 'List registered packages'
     usage = \
-'''%prog
+'''%prog {index-spec}
 
 List registered packages.
 '''
     def run(self, options, args):
-        repo = self._get_repo()
-        for pkg in repo.index.list():
-            print pkg.name
+        spec_from = args[0]
+        index, path = self.index_from_spec(spec_from)
+        for pkg in index.list():
+            print u'%s -- %s' % (pkg.name, pkg.title)
 
 ListCommand()
 
 
+class SearchCommand(Command):
+    name = 'search'
+    summary = 'Search registered packages'
+    usage = \
+'''%prog {index-spec} {query}
+
+Search registered packages in index-spec.
+'''
+    def run(self, options, args):
+        spec_from = args[0]
+        if len(args) > 1:
+            query = args[1]
+        else:
+            query = ''
+        index, path = self.index_from_spec(spec_from)
+        for pkg in index.search(query):
+            print u'%s -- %s' % (pkg.name, pkg.title)
+
+SearchCommand()
+
+
 class InfoCommand(Command):
     name = 'info'
     summary = 'Get information about a package'
     usage = \
-'''%prog {path-or-pkg-name}
+'''%prog {package-spec} [manifest]
 
-Get information about a package.
+Get information about a package (print package metadata). If manifest specified
+then show manifest info rather than package metadata.
+
+WARNING: if you change the metadata for a python distribution you may need to
+rebuild the egg-info for changes to show up here.
 '''
 
     def run(self, options, args):
-        pkg_locator = args[0]
-        pkg = self._get_package(pkg_locator)
+        spec_from = args[0]
+        index, path = self.index_from_spec(spec_from)
+        pkg = index.get(path)
         if pkg is None:
-            print 'No package was found for: "%s"' % pkg_locator
+            print 'No package was found for: "%s"' % spec_from
             return 1
-        self._print_pkg(pkg)
+        if len(args) > 1 and args[1] == 'manifest':
+            print '### Manifest\n'
+            for resource in pkg.manifest:
+                print resource
+        else:
+            self._print_pkg(pkg)
 
 InfoCommand()
 
@@ -292,10 +364,10 @@ Dump contents of specified resource in specified package to stdout.
 '''
 
     def run(self, options, args):
-        # assuming no spaces in the paths!
-        pkg_locator = args[0]
+        spec_from = args[0]
+        index, path = self.index_from_spec(spec_from)
+        pkg = index.get(path)
         offset = args[1]
-        pkg = self._get_package(pkg_locator)
         stream = pkg.stream(offset)
         import sys
         sys.stdout.write(stream.read()) 
@@ -303,21 +375,17 @@ Dump contents of specified resource in specified package to stdout.
 DumpCommand()
 
 
-# def do_inspect(self, line):
-    # pkg_locator = line.strip()
-    # pkg = self._get_package(pkg_locator)
-    # print pkg.distribution.listdir
-
-
-class RepoCommand(Command):
-    name = 'repo'
-    summary = 'Manage a repository'
+class InitCommand(Command):
+    name = 'init'
+    summary = 'Initialise things (config, repository etc)'
     usage = '''%prog {action}
 
-init: Initialize a repository.
+config: Create configuration file at default location (see --config)
+
+repo: Initialize a repository. The repository will be created at the location
+      specified via the --repository option or default location specified by
+      config.
 
-    The repository will be created at the location specified via the
-    --repository option or default location if not specified (~/.datapkg).
 '''
 
     def run(self, options, args):
@@ -329,14 +397,19 @@ init: Initialize a repository.
         else:
             print 'You must supply an action'
     
-    def init(self, args, options):
+    def repo(self, args, options):
         import datapkg.repository
         repo = datapkg.repository.Repository(self.repository_path)
         repo.init()
         msg = 'Repository successfully initialized at %s' % self.repository_path
         self._print(msg)
     
-RepoCommand()
+    def config(self, args, options):
+        '''Create default config file.'''
+        import datapkg.config
+        cfg = datapkg.config.write_default_config()
+    
+InitCommand()
 
 
 class CreateCommand(Command):
@@ -360,11 +433,11 @@ directory.'''
  
     def run(self, options, args):
         path = args[0]
-        import datapkg.package
+        import datapkg.distribution
         template = options.template 
         msg = 'Creating new datapkg: %s' %  path
         self._print(msg)
-        datapkg.package.PackageMaker.create_on_disk(path, template=template)
+        datapkg.package.Package.create_on_disk(path, template=template)
 
 CreateCommand()
 
@@ -373,36 +446,76 @@ class RegisterCommand(Command):
     name = 'register'
     summary = 'Register a package'
     usage = \
-'''%prog {path}
+'''%prog {src-spec} {dest-spec}
 
-Register package at path in the in the local index.
+Register package at {src-spec} into index at {dest-spec}.
 '''
 
     def run(self, options, args):
-        path = args[0]
-        self._register(path)
+        spec_from = args[0]
+        spec_to = args[1]
+        index, path = self.index_from_spec(spec_from)
+        index_to, path_to = self.index_from_spec(spec_to)
+        pkg = index.get(path)
+        index_to.register(pkg)
 
 RegisterCommand()
 
 
+class UpdateCommand(Command):
+    name = 'update'
+    summary = 'Update a package'
+    usage = \
+'''%prog {src-spec} {dest-spec}
+
+As for register.
+'''
+
+    def run(self, options, args):
+        spec_from = args[0]
+        spec_to = args[1]
+        index, path = self.index_from_spec(spec_from)
+        index_to, path_to = self.index_from_spec(spec_to)
+        pkg = index.get(path)
+        index_to.update(pkg)
+
+UpdateCommand()
+
+
 class InstallCommand(Command):
     name = 'install'
     summary = 'Install a package'
     usage = \
-'''%prog {path}
+'''%prog {src-spec} {dest-spec}
 
-Install a package located at path on disk.
+Install a package located {src-spec} to {dest-spec}, e.g.::
+
+    install ckan://name path-on-disk
 '''
 
     def run(self, options, args):
-        pkg_path = args[0]
-        # TODO: check whether it is registered already
-        # TODO: option of installing an existing registered package
-        repo, pkg = self._register(pkg_path)
-        install_dir = repo.installed_path
-        pkg.install(install_dir, pkg_path)
-        # TODO: save package again as e.g. installed path may have changed
-        # This should probably be moved down into package object
+        spec_from = args[0]
+        spec_to = args[1]
+        index, path = self.index_from_spec(spec_from)
+        index_to, path_to = self.index_from_spec(spec_to)
+        pkg = index.get(path)
+        # TODO: have to reimport here for this to work. Why?
+        import datapkg.index
+        if not isinstance(index_to, datapkg.index.FileIndex):
+            msg = u'You can only install to the local filesystem'
+            raise Exception(msg)
+        if pkg.installed_path: # on local filesystem ...
+            import shutil
+            dest = os.path.join(path_to, pkg.name)
+            shutil.copytree(pkg.installed_path, dest)
+        elif pkg.download_url:
+            import datapkg.util
+            downloader = datapkg.util.Downloader(path_to)
+            downloader.download(pkg.download_url)
+        else:
+            msg = u'Cannot install from package %s (relevant info not present)' % pkg.name
+            raise Exception(msg)
+        # pkg.install
 
 InstallCommand()
 
